﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.IO

Class MainWindow
    Implements INotifyPropertyChanged

    Private Sub MainWindow_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded

        AllButtons = New ObservableCollection(Of PlayStoneBase)

        MixStonesCommand.Execute(Nothing)
        Me.DataContext = Me
    End Sub


    Private _allButtons As ObservableCollection(Of PlayStoneBase)
    Public Property AllButtons() As ObservableCollection(Of PlayStoneBase)
        Get
            Return _allButtons
        End Get
        Set(ByVal value As ObservableCollection(Of PlayStoneBase))
            _allButtons = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(AllButtons)))
        End Set
    End Property


    Private ReadOnly Property Rows As Integer
        Get
            Return FieldSize
        End Get
    End Property


    Private ReadOnly Property Columns As Integer
        Get
            Return FieldSize
        End Get
    End Property


    Private _fieldSize As Integer = 4
    Public Property FieldSize() As Integer
        Get
            Return _fieldSize
        End Get
        Set(ByVal value As Integer)
            _fieldSize = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(FieldSize)))
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(Rows)))
            MixStonesCommand.Execute(Nothing)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(Columns)))
        End Set
    End Property



    Private _moveButtonCommand As ICommand
    Public Property MoveButtonCommand() As ICommand
        Get
            If _moveButtonCommand Is Nothing Then _
                    _moveButtonCommand = New RelayCommand(AddressOf MoveButtonCommand_Execute,
                                                               AddressOf MoveButtonCommand_CanExecute)
            Return _moveButtonCommand
        End Get
        Set(ByVal value As ICommand)
            _moveButtonCommand = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(MoveButtonCommand)))
        End Set
    End Property


    Private Function MoveButtonCommand_CanExecute(obj As Object) As Boolean
        Return True
    End Function


    Private Sub MoveButtonCommand_Execute(obj As Object)
        Dim currIndex = AllButtons.IndexOf(DirectCast(obj, PlayButton))
        Dim placeholder = AllButtons.Where(Function(x) x.GetType Is GetType(Placeholder)).Single
        Dim emptyIndex = AllButtons.IndexOf(placeholder)

        'Liegt Stein unmittelbar neben dem Placeholder?
        If (emptyIndex - Columns) = currIndex OrElse (emptyIndex + Columns) = currIndex OrElse currIndex - emptyIndex = 1 OrElse emptyIndex - currIndex = 1 Then
            AllButtons.Move(currIndex, emptyIndex)
            AllButtons.Remove(placeholder)
            AllButtons.Insert(currIndex, New Placeholder())
        End If
    End Sub


    Private _mixStonesCommand As ICommand
    Public Property MixStonesCommand() As ICommand
        Get
            If _mixStonesCommand Is Nothing Then _
                _mixStonesCommand = New RelayCommand(AddressOf MixStonesCommand_Execute,
                                             AddressOf MixStonesCommand_CanExecute)
            Return _mixStonesCommand
        End Get
        Set(ByVal value As ICommand)
            _mixStonesCommand = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(MixStonesCommand)))
        End Set
    End Property

    Private Function MixStonesCommand_CanExecute(obj As Object) As Boolean
        Return True
    End Function

    Private Sub MixStonesCommand_Execute(obj As Object)
        AllButtons.Clear()

        For i As Integer = 0 To (Columns * Rows) - 2
            AllButtons.Add(New PlayButton With {.Number = i + 1})
        Next
        AllButtons.Add(New Placeholder())
        AllButtons = New ObservableCollection(Of PlayStoneBase)(Shuffle(Of PlayStoneBase)(AllButtons))
    End Sub

    Function Shuffle(Of T)(collection As IEnumerable(Of T)) As List(Of T)
        Dim r As Random = New Random()
        Shuffle = collection.OrderBy(Function(a) r.Next()).ToList()
    End Function


    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class
